# Internationalization (I18N) - Language Translations

## Overview

This document provides a brief overview of changes needed when adding new languages to FastSitePHP. Three different repositories need to be updated for full site translations. If you would like to help with translations feel free to open an issue or simply submit translation changes.

This document itself does NOT need to be translated rather it specifies what files and code should be updated.

The following setup is recommended for local development.

~~~
fastsitephp     {root-directory}
├── fastsitephp   [repository]
├── playground    [repository]
└── starter-site  [repository]
~~~

If you need to install PHP see documents for (Windows, macOS, and Linux) from: https://www.fastsitephp.com/en/getting-started

Files are copied from English, Google Translate or other sites are used for initial translations, and then human translators fix errors from machine translations.

Languages currently needed: Arabic, French, German, Italian, Japanese, Persian, Russian

Languages currently being tranlsated: Chinese Simplified (zh-CN), Chinese Traditional (zh-TW), Korean, Portuguese

## Generating the files

All needed files can be generated by using the php script once all 3 repositories are downloaded:

https://github.com/fastsitephp/fastsitephp/blob/master/scripts/create-i18n-files.php

Simply change the value for [const LANG_COPY_TO = '{lang}';] on line 18 and run it.

## JSON Files

Files for specific languages end with `*.{lang}.json` and the main file for each site is `_.{lang}.json`.

* `fastsitephp/website/app_data/i18n`
* `starter-site/app_data/i18n`

https://github.com/fastsitephp/fastsitephp/tree/master/website/app_data/i18n

https://github.com/fastsitephp/starter-site/tree/master/app_data/i18n

## Related Nav Menu

Search the following file for `i18n-menu` and `$github`. There are 2 `i18n-menu` menus (one desktop and one mobile). `$github` is used to specify the readme file.

https://github.com/fastsitephp/fastsitephp/blob/master/website/app/Views/_header.php

The order of languages for the Nav Menu is based on alphabetical sort order, this can be determined by copying the text to Excel and sorting.

For a good link on lanuages used on the internet see: https://en.wikipedia.org/wiki/Languages_used_on_the_Internet

## Code Playground Template

Each language has it's own template directory/folder. Once a directory is created for a language then it can be used for to create sites. Text content and code comments are updated to the translated language.

`playground\app_data\template\{lang}`

https://github.com/fastsitephp/playground/tree/master/app_data/template

This can be done without a local setup, but if you would prefer to test the site locally refer to instructions in the readme of the playground repository.

https://github.com/fastsitephp/playground

## Home Page Code

Code comments are updated and some text is changed `home-page-{lang}.php`

https://github.com/fastsitephp/fastsitephp/tree/master/website/app_data/sample-code

## Main Site Readme

Some of this content is duplicated from the main home page so some content simply be copied after translating JSON files. The 'Paid Translators Needed' section should be manually deleted on the copied file.

`fastsitephp\docs\i18n-readme\README.{lang}.md`

https://github.com/fastsitephp/fastsitephp/tree/master/docs/i18n-readme

## Starter Site Readme

Each translated language will have it’s own starter site readme file.

`starter-site\docs\README.{lang}.md`

https://github.com/fastsitephp/starter-site/blob/master/README.md

https://github.com/fastsitephp/starter-site/tree/master/docs

## Additional Readme updates

Readme files for the `playground` and `static-files` repositories will have a single row added to the table at the top of the file. Copy and tanslate the English row.

https://github.com/fastsitephp/playground/blob/master/README.md

https://github.com/fastsitephp/static-files/blob/master/README.md

## Code for the Example Pages

Update comments in `*.{lang}.txt` files

https://github.com/fastsitephp/fastsitephp/tree/master/website/app_data/i18n/code

## Quick Reference Code

Code comments are updated and some text is changed `home-page-{lang}-examples.php`. This is a large file so tranlations are planned for the future after many languages are first translated. Additionally `home-page-{lang}.php` should load the new file when created, this is specified at the bottom of each `home-page` file for the language.

https://github.com/fastsitephp/fastsitephp/tree/master/website/app_data/sample-code


## Sitemap Update

Once content is ready the `sitemap.xml` file can be updated by following instructions in the Sitemap Controller file.

https://github.com/fastsitephp/fastsitephp/blob/master/website/app/Controllers/Sitemap.php

## Docs and API

These file are also planned for the future due to the amount of content. The API docs will need a custom UI created in order to edit them as much of the JSON content contains line breaks and formatting.

https://github.com/fastsitephp/fastsitephp/tree/master/website/app_data/docs/en

https://github.com/fastsitephp/fastsitephp/tree/master/website/app_data/api
